@@ -2,9 +2,10 @@
 
 set -euo pipefail
 
-INTEGRATIONTESTS=1
-UNITTESTS=1
-DEPCHECK=1
+INTEGRATION_TESTS=1
+UNIT_TESTS=1
+CODE_CHECK=1
+DOC_CHECK=1
 GEN_UNIT=0
 GEN_TOLERANCE=0
 SKIP_PIP=0
@@ -18,43 +19,59 @@ if [ -z "${FAUCET_TESTS:-}" ]; then
   FAUCET_TESTS="$*"
 fi
 
-# Shorten long form arguments to make parsing easier
-FAUCET_TESTS_SHORTENED=""
-for opt in ${FAUCET_TESTS}; do
-  case "${opt}" in
-    --integration)
-      FAUCET_TESTS_SHORTENED+=" -i"
-      ;;
-    --nocheck)
-      FAUCET_TESTS_SHORTENED+=" -n"
-      ;;
-    *)
-      FAUCET_TESTS_SHORTENED+=" ${opt}"
-      ;;
-  esac
-done
-
 PARAMS=""
 
 # Parse options, some are used by this script, some are
 # passed onto mininet_main.py & clib_mininet_main.py
-for opt in ${FAUCET_TESTS_SHORTENED}; do
+for opt in ${FAUCET_TESTS}; do
   case "${opt}" in
     --help)
       HELP=1
       ;;
+    --check)
+      INTEGRATION_TESTS=0
+      UNIT_TESTS=0
+      CODE_CHECK=1
+      DOC_CHECK=1
+      ;;
+    --integration)
+      INTEGRATION_TESTS=1
+      UNIT_TESTS=0
+      CODE_CHECK=0
+      DOC_CHECK=0
+      PARAMS+=" -i"		# Is this still needed ?
+      ;;
+    --unit)
+      INTEGRATION_TESTS=0
+      UNIT_TESTS=1
+      CODE_CHECK=0
+      DOC_CHECK=0
+      ;;
+    --nocheck)
+      CODE_CHECK=0
+      DOC_CHECK=0
+      PARAMS+=" -n"		# Is this still needed ?
+      ;;
+    --nointegration)
+      INTEGRATION_TESTS=0
+      ;;
+    --nounit)
+      UNIT_TESTS=0
+      ;;
     --generative_unit)
       GEN_UNIT=1
-      UNITTESTS=0
-      DEPCHECK=0
-      INTEGRATIONTESTS=0
+      UNIT_TESTS=0
+      CODE_CHECK=0
+      DOC_CHECK=0
+      INTEGRATION_TESTS=0
       PARAMS+=" ${opt}"
       ;;
     --generative_tolerance)
       GEN_TOLERANCE=1
-      UNITTESTS=0
-      DEPCHECK=0
-      INTEGRATIONTESTS=0
+      UNIT_TESTS=0
+      CODE_CHECK=0
+      DOC_CHECK=0
+      INTEGRATION_TESTS=0
       PARAMS+=" ${opt}"
       ;;
     --*)
@@ -65,18 +82,20 @@ for opt in ${FAUCET_TESTS_SHORTENED}; do
         case "${opt:$i:1}" in
           i)
             # run only integration tests
-            UNITTESTS=0
-            DEPCHECK=0
+            UNIT_TESTS=0
+            CODE_CHECK=0
+            DOC_CHECK=0
             PARAMS+=" -${opt:$i:1}"
             ;;
           n)
-            # skip code check
-            DEPCHECK=0
+            # skip code and docs chec
+            CODE_CHECK=0
+            DOC_CHECK=0
             PARAMS+=" -${opt:$i:1}"
             ;;
           u)
             # skip unit tests
-            UNITTESTS=0
+            UNIT_TESTS=0
             ;;
           z)
             # Skip pip installer
@@ -106,14 +125,62 @@ if [ "$SKIP_PIP" == 0 ] ; then
     echo "Using pip cache"
     pip_deps_args+=("--pip-args=--cache-dir=/var/tmp/pip-cache")
   fi
-  if [ "$DEPCHECK" == 1 ] ; then
+  if [ "$CODE_CHECK" == 1 ] ; then
     pip_deps_args+=("--extra-requirements=codecheck-requirements.txt")
   fi
+  if [ "$DOC_CHECK" == 1 ] ; then
+    pip_deps_args+=("--extra-requirements=docs/requirements.txt")
+  fi
   ./docker/pip_deps.sh "${pip_deps_args[@]}"
 else
   echo "Skipping pip install script"
 fi
 
+export PYTHONPATH=/faucet-src:/faucet-src/faucet:/faucet-src/clib
+
+if [ "$HELP" == 1 ] ; then
+  cd /faucet-src/tests/integration
+  ./mininet_main.py --help
+  exit 0
+fi
+
+if [ "$UNIT_TESTS" == 1 ] ; then
+  echo "========== Running faucet unit tests =========="
+  cd /faucet-src/tests
+  time ./run_unit_tests.sh
+elif [ "$GEN_UNIT" == 1 ] ; then
+  echo "========== Running faucet generative unit tests =========="
+  cd /faucet-src/tests/generative/unit/
+  time ./test_topology.py
+else
+  echo "========== Skipping unit tests =========="
+fi
+
+if [ "$CODE_CHECK" == 1 ] ; then
+  cd /faucet-src/tests/codecheck
+  echo "============ Running pylint analyzer ============"
+  time ./pylint.sh $PY_FILES_CHANGED
+  echo "============ Running flake8 analyzer ============"
+  time ./flake8.sh $PY_FILES_CHANGED
+  echo "============ Running pytype analyzer ============"
+  time ./pytype.sh $PY_FILES_CHANGED
+else
+  echo "========== Skipping code checks =========="
+fi
+
+if [ "$DOC_CHECK" == 1 ] ; then
+  echo "========== Building documentation =========="
+  cd /faucet-src/docs
+  time make html
+  rm -rf _build
+fi
+
+if [ "$INTEGRATION_TESTS" == 0 ] && [ "$GEN_TOLERANCE" == 0 ] ; then
+  echo "========== Skipping integration tests =========="
+  echo Done with faucet system tests.
+  exit 0
+fi
+
 echo "========== checking IPv4/v6 localhost is up ====="
 ping6 -c 1 ::1
 ping -c 1 127.0.0.1
@@ -126,41 +193,7 @@ ovs-vsctl --no-wait set Open_vSwitch . other_config:max-idle=50000
 # Needed to support double tagging.
 ovs-vsctl --no-wait set Open_vSwitch . other_config:vlan-limit=2
 
-cd /faucet-src/tests
-
-./sysctls_for_tests.sh || true
-
-export PYTHONPATH=/faucet-src:/faucet-src/faucet:/faucet-src/clib
-
-if [ "$HELP" == 1 ] ; then
-  cd /faucet-src/tests/integration
-  ./mininet_main.py --help
-  exit 0
-fi
-
-if [ "$UNITTESTS" == 1 ] ; then
-  echo "========== Running faucet unit tests =========="
-  cd /faucet-src/tests
-  time ./run_unit_tests.sh
-elif [ "$GEN_UNIT" == 1 ] ; then
-  echo "========== Running faucet generative unit tests =========="
-  cd /faucet-src/tests/generative/unit/
-  ./test_topology.py
-  cd /faucet-src/tests
-fi
-
-if [ "$DEPCHECK" == 1 ] ; then
-  echo "========== Building documentation =========="
-  cd /faucet-src/docs
-  time make html
-  rm -rf _build
-
-  cd /faucet-src/tests/codecheck
-  echo "============ Running pylint analyzer ============"
-  time ./pylint.sh $PY_FILES_CHANGED
-  echo "============ Running pytype analyzer ============"
-  time ./pytype.sh $PY_FILES_CHANGED
-fi
+/faucet-src/tests/sysctls_for_tests.sh || true
 
 echo "========== Starting docker container =========="
 mkdir -p /var/local/run/
@@ -177,7 +210,7 @@ test_failures=
 export FAUCET_DIR=/faucet-src/faucet
 export http_proxy=
 
-if [ "$INTEGRATIONTESTS" == 1 ] ; then
+if [ "$INTEGRATION_TESTS" == 1 ] ; then
   echo "========== Running faucet integration tests =========="
   cd /faucet-src/tests/integration
   ./mininet_main.py -c
@@ -227,7 +260,7 @@ EOL
   ovs-ofctl dump-ports hwbr
 fi
 
-if [ "$INTEGRATIONTESTS" == 1 ]; then
+if [ "$INTEGRATION_TESTS" == 1 ]; then
   time ./mininet_main.py $FAUCET_TESTS || test_failures+=" mininet_main"
   cd /faucet-src/clib
   time ./clib_mininet_test.py $FAUCET_TESTS || test_failures+=" clib_mininet_test"
