@@ -23,7 +23,7 @@ from faucet import valve_of
 from faucet.faucet_pipeline import ValveTableConfig
 
 
-class ValveTable: # pylint: disable=too-many-arguments,too-many-instance-attributes
+class ValveTable:  # pylint: disable=too-many-arguments,too-many-instance-attributes
     """Wrapper for an OpenFlow table."""
 
     def __init__(self, name, table_config,
@@ -92,7 +92,7 @@ class ValveTable: # pylint: disable=too-many-arguments,too-many-instance-attribu
     # TODO: verify actions
     @staticmethod
     @functools.lru_cache(maxsize=1024)
-    def match(in_port=None, vlan=None, # pylint: disable=too-many-arguments,too-many-locals
+    def match(in_port=None, vlan=None,  # pylint: disable=too-many-arguments
               eth_type=None, eth_src=None, eth_dst=None, eth_dst_mask=None,
               icmpv6_type=None, nw_proto=None, nw_dst=None, metadata=None,
               metadata_mask=None, vlan_pcp=None, udp_src=None, udp_dst=None):
@@ -143,7 +143,9 @@ class ValveTable: # pylint: disable=too-many-arguments,too-many-instance-attribu
         set_fields = {action.key for action in new_actions if valve_of.is_set_field(action)}
         if self.table_id != valve_of.ofp.OFPTT_ALL and set_fields:
             assert set_fields.issubset(self.set_fields), (
-                'unexpected set fields %s configured %s in %s' % (set_fields, self.set_fields, self.name))
+                'unexpected set fields %s configured %s in %s' % (set_fields,
+                                                                  self.set_fields,
+                                                                  self.name))
         return new_actions
 
     @functools.lru_cache()
@@ -155,8 +157,12 @@ class ValveTable: # pylint: disable=too-many-arguments,too-many-instance-attribu
             new_inst = []
             for instruction in inst:
                 if instruction.type == valve_of.ofp.OFPIT_APPLY_ACTIONS:
+                    recirc_present = any((
+                        True for action in instruction.actions
+                        if valve_of.is_ct(action) and hasattr(action, 'recirc_table')
+                    ))
                     # If no goto present, this is the last set of actions that can take place
-                    if not goto_present:
+                    if not goto_present and not recirc_present:
                         instruction.actions = self._trim_actions(instruction.actions)
                     # Always drop an apply actions instruction with no actions.
                     if not instruction.actions:
@@ -165,12 +171,12 @@ class ValveTable: # pylint: disable=too-many-arguments,too-many-instance-attribu
             return tuple(new_inst)
         return inst
 
-    def flowmod(self, match=None, priority=None, # pylint: disable=too-many-arguments
+    def flowmod(self, match=None, priority=None,  # pylint: disable=too-many-arguments
                 inst=None, command=valve_of.ofp.OFPFC_ADD, out_port=0,
                 out_group=0, hard_timeout=0, idle_timeout=0, cookie=None):
         """Helper function to construct a flow mod message with cookie."""
         if priority is None:
-            priority = 0 # self.dp.lowest_priority
+            priority = 0  # self.dp.lowest_priority
         if not match:
             match = self.match()
         if inst is None:
@@ -255,8 +261,7 @@ class ValveGroupEntry:
 class ValveGroupTable:
     """Wrap access to group table."""
 
-    entries = None # type: dict
-
+    entries = None  # type: dict
 
     def __init__(self):
         """Constructs a new object"""
